@@ -10,6 +10,7 @@ import os
 import subprocess
 import re
 import logging
+from functools import partial
 from tryton.config import CONFIG
 from tryton.config import TRYTON_ICON, PIXMAPS_DIR
 import time
@@ -248,8 +249,6 @@ def refresh_langlist(lang_widget, host, port):
         lang_list = rpc.db_exec(host, port, 'list_lang')
     except socket.error:
         return []
-    from tryton.gui.main import Main
-    Main.get_main().refresh_ssl()
     index = -1
     i = 0
     lang = locale.getdefaultlocale()[0]
@@ -971,9 +970,10 @@ def send_bugtracker(title, msg):
             protocol = 'http'
             if ssl or hasattr(socket, 'ssl'):
                 protocol = 'https'
+            quote = partial(urllib.quote, safe="!$&'()*+,;=:")
             server = xmlrpclib.Server(
                 ('%s://%s:%s@' + CONFIG['roundup.xmlrpc'])
-                % (protocol, user, password), allow_none=True)
+                % (protocol, quote(user), quote(password)), allow_none=True)
             if hashlib:
                 msg_md5 = hashlib.md5(msg + '\n' + title).hexdigest()
             else:
@@ -1104,7 +1104,6 @@ def process_exception(exception, *args, **kwargs):
                         return False
                     res = rpc.login(rpc._USERNAME, password, hostname, port,
                             rpc._DATABASE)
-                    Main.get_main().refresh_ssl()
                     if res == -1:
                         message(_('Connection error!\n'
                                 'Unable to connect to the server!'))
@@ -1194,20 +1193,18 @@ def generateColorscheme(masterColor, keys, light=0.06):
 class DBProgress(object):
 
     def __init__(self, host, port):
-        self.dbs, self.createdb = None, None
+        self.dbs = None, None
         self.host, self.port = host, port
         self.updated = threading.Event()
-        self.db_info = None
 
     def start(self):
-        dbs, createdb = None, False
+        dbs = None
         try:
             dbs = refresh_dblist(self.host, self.port)
-            createdb = True
         except Exception:
             pass
         finally:
-            self.db_info = (dbs, createdb)
+            self.dbs = dbs
             self.updated.set()
 
     def update(self, combo, progressbar, callback, dbname=''):
@@ -1222,15 +1219,9 @@ class DBProgress(object):
             progressbar.pulse()
             return True
         progressbar.hide()
-        dbs, createdb = self.db_info
+        dbs = self.dbs
 
-        if dbs is None:
-            dbs, createdb = None, False
-        elif dbs == -1:
-            dbs, createdb = -1, False
-        else:
-            from tryton.gui.main import Main
-            Main.get_main().refresh_ssl()
+        if dbs is not None and dbs not in (-1, -2):
             liststore = combo.get_model()
             liststore.clear()
             index = -1
@@ -1243,7 +1234,7 @@ class DBProgress(object):
             combo.set_active(index)
             dbs = len(dbs)
 
-        callback(dbs, createdb)
+        callback(dbs)
         return False
 
 
