@@ -4,25 +4,19 @@
 import gettext
 import gtk
 import gobject
-import locale
-import tryton.rpc as rpc
 from tryton.gui.window.view_form.screen import Screen
 from tryton.action import Action
-from tryton.config import CONFIG
 from tryton.gui import Main
 from tryton.gui.window import Window
-from tryton.gui.window.preference import Preference
 from tryton.gui.window.win_export import WinExport
 from tryton.gui.window.win_import import WinImport
 from tryton.gui.window.attachment import Attachment
 from tryton.signal_event import SignalEvent
-from tryton.common import (TRYTON_ICON, message, sur, sur_3b, COLOR_SCHEMES,
-    timezoned_date)
-from tryton.exceptions import TrytonServerError
+from tryton.common import message, sur, sur_3b, COLOR_SCHEMES, timezoned_date
 import tryton.common as common
 from tryton.translate import date_format
-from tryton.common import DT_FORMAT, DHM_FORMAT, HM_FORMAT
-from tryton.pyson import PYSONEncoder
+from tryton.common import RPCExecute, RPCException
+from tryton import plugins
 
 from tabcontent import TabContent
 
@@ -53,30 +47,31 @@ class Form(SignalEvent, TabContent):
     menu_def = [
         (_('_New'), 'tryton-new', 'sig_new', '<tryton>/Form/New'),
         (_('_Save'), 'tryton-save', 'sig_save', '<tryton>/Form/Save'),
+        (_('_Switch View'), 'tryton-fullscreen', 'sig_switch',
+            '<tryton>/Form/Switch View'),
+        (_('_Reload/Undo'), 'tryton-refresh', 'sig_reload',
+            '<tryton>/Form/Reload'),
         (_('_Duplicate'), 'tryton-copy', 'sig_copy',
             '<tryton>/Form/Duplicate'),
         (_('_Delete...'), 'tryton-delete', 'sig_remove',
             '<tryton>/Form/Delete'),
         (None,) * 4,
-        (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
         (_('_Previous'), 'tryton-go-previous', 'sig_previous',
             '<tryton>/Form/Previous'),
-        (_('_Switch View'), 'tryton-fullscreen', 'sig_switch',
-            '<tryton>/Form/Switch View'),
+        (_('_Next'), 'tryton-go-next', 'sig_next', '<tryton>/Form/Next'),
+        (_('_Search'), 'tryton-find', 'sig_search', '<tryton>/Form/Search'),
+        (_('View _Logs...'), None, 'sig_logs', None),
         (None,) * 4,
         (_('_Close Tab'), 'tryton-close', 'sig_win_close',
             '<tryton>/Form/Close'),
         (None,) * 4,
-        (_('View _Logs...'), None, 'sig_logs', None),
-        (_('_Go to Record ID...'), None, 'sig_goto', '<tryton>/Form/Goto'),
-        (None,) * 4,
-        (_('_Reload/Undo'), 'tryton-refresh', 'sig_reload',
-            '<tryton>/Form/Reload'),
-        (None,) * 4,
+        (_('A_ttachments...'), 'tryton-attachment', 'sig_attach',
+            '<tryton>/Form/Attachments'),
         (_('_Actions...'), 'tryton-executable', 'sig_action',
             '<tryton>/Form/Actions'),
         (_('_Relate...'), 'tryton-go-jump', 'sig_relate',
             '<tryton>/Form/Relate'),
+        (None,) * 4,
         (_('_Report...'), 'tryton-print-open', 'sig_print_open',
             '<tryton>/Form/Report'),
         (_('_E-Mail...'), 'tryton-print-email', 'sig_print_email',
@@ -90,9 +85,9 @@ class Form(SignalEvent, TabContent):
             '<tryton>/Form/Import Data'),
     ]
 
-    def __init__(self, model, res_id=False, domain=None, mode=None,
+    def __init__(self, model, res_id=False, domain=None, order=None, mode=None,
             view_ids=None, context=None, name=False, limit=None,
-            auto_refresh=False, search_value=None):
+            auto_refresh=False, search_value=None, tab_domain=None):
         super(Form, self).__init__()
 
         if not mode:
@@ -114,8 +109,9 @@ class Form(SignalEvent, TabContent):
         self.dialogs = []
 
         self.screen = Screen(self.model, mode=mode, context=self.context,
-            view_ids=view_ids, domain=domain, limit=limit,
-            readonly=bool(auto_refresh), search_value=search_value)
+            view_ids=view_ids, domain=domain, limit=limit, order=order,
+            readonly=bool(auto_refresh), search_value=search_value,
+            tab_domain=tab_domain)
         self.screen.widget.show()
 
         if not name:
@@ -124,10 +120,26 @@ class Form(SignalEvent, TabContent):
             self.name = name
 
         self.create_tabcontent()
+
+        self.url_entry = url_entry = gtk.Entry()
+        url_entry.show()
+        url_entry.set_editable(False)
+        style = url_entry.get_style()
+        url_entry.modify_bg(gtk.STATE_ACTIVE,
+            style.bg[gtk.STATE_INSENSITIVE])
+        self.widget.pack_start(url_entry, False, False)
+
+        access = common.MODELACCESS[self.model]
+        for button, access_type in (
+                ('new', 'create'),
+                ('save', 'write'),
+                ):
+            self.buttons[button].props.sensitive = access[access_type]
+
         self.screen.signal_connect(self, 'record-message',
             self._record_message)
         self.screen.signal_connect(self, 'record-modified',
-            self._record_modified)
+            lambda *a: gobject.idle_add(self._record_modified, *a))
         self.screen.signal_connect(self, 'record-saved', self._record_saved)
         self.screen.signal_connect(self, 'attachment-count',
                 self._attachment_count)
@@ -147,16 +159,11 @@ class Form(SignalEvent, TabContent):
             gobject.timeout_add(int(auto_refresh) * 1000, self.sig_reload)
 
     def get_toolbars(self):
-        ctx = {}
-        ctx.update(rpc.CONTEXT)
-        ctx.update(self.context)
-        args = ('model', self.model, 'view_toolbar_get', ctx)
         try:
-            toolbars = rpc.execute(*args)
-        except TrytonServerError, exception:
-            toolbars = common.process_exception(exception, *args)
-            toolbars = toolbars if toolbars else {}
-        return toolbars
+            return RPCExecute('model', self.model, 'view_toolbar_get',
+                context=self.context)
+        except RPCException:
+            return {}
 
     def widget_get(self):
         return self.screen.widget
@@ -177,50 +184,7 @@ class Form(SignalEvent, TabContent):
             and self.auto_refresh == value.auto_refresh
             and self.screen.search_value == value.screen.search_value)
 
-    def sig_goto(self, widget=None):
-        if not self.modified_save():
-            return
-        parent = common.get_toplevel_window()
-        win = gtk.Dialog(_('Go to ID'), parent,
-                gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
-                (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                    gtk.STOCK_OK, gtk.RESPONSE_OK))
-        win.set_icon(TRYTON_ICON)
-        win.set_has_separator(True)
-        win.set_default_response(gtk.RESPONSE_OK)
-
-        table = gtk.Table(2, 2)
-        table.attach(gtk.Label(_('Go to ID:')), 1, 2, 0, 1, gtk.FILL)
-        img = gtk.Image()
-        img.set_from_stock('tryton-go-jump', gtk.ICON_SIZE_DIALOG)
-        table.attach(img, 0, 1, 0, 2, gtk.FILL)
-
-        entry = gtk.Entry()
-        entry.set_property('activates_default', True)
-        entry.set_max_length(0)
-        entry.set_alignment(1.0)
-        def sig_insert_text(widget, new_text, new_text_length, position):
-            value = widget.get_text()
-            position = widget.get_position()
-            new_value = value[:position] + new_text + value[position:]
-            try:
-                locale.atoi(new_value)
-            except ValueError:
-                widget.stop_emission('insert-text')
-        entry.connect('insert_text', sig_insert_text)
-        table.attach(entry, 1, 2, 1, 2)
-
-        win.vbox.pack_start(table, expand=True, fill=True)
-        win.show_all()
-
-        response = win.run()
-        if response == gtk.RESPONSE_OK:
-            self.screen.display(locale.atoi(entry.get_text()), set_cursor=True)
-        win.destroy()
-        parent.present()
-
     def destroy(self):
-        self.screen.signal_unconnect(self)
         self.screen.destroy()
         self.screen = None
         self.widget = None
@@ -237,13 +201,11 @@ class Form(SignalEvent, TabContent):
         return self.screen.id_get()
 
     def sig_attach(self, widget=None):
-        obj_id = self.id_get()
-        if obj_id >= 0 and obj_id is not False:
-            Attachment(self.model, obj_id).run()
-        else:
-            self.message_info(_('No record selected!'))
-        self.update_attachment_count(reload=True)
-        return True
+        record = self.screen.current_record
+        if not record or record.id < 0:
+            return
+        Attachment(record,
+            lambda: self.update_attachment_count(reload=True))
 
     def update_attachment_count(self, reload=False):
         record = self.screen.current_record
@@ -260,6 +222,9 @@ class Form(SignalEvent, TabContent):
             self.buttons['attach'].set_stock_id('tryton-attachment-hi')
         else:
             self.buttons['attach'].set_stock_id('tryton-attachment')
+        record_id = self.id_get()
+        self.buttons['attach'].props.sensitive = bool(
+            record_id >= 0 and record_id is not False)
 
     def sig_switch(self, widget=None):
         if not self.modified_save():
@@ -280,31 +245,28 @@ class Form(SignalEvent, TabContent):
             ('write_date', _('Latest Modification Date:')),
         ]
 
-        ctx = self.context.copy()
-        ctx.update(rpc.CONTEXT)
-        args = ('model', self.model, 'read', [obj_id], [x[0] for x in fields],
-                ctx)
         try:
-            res = rpc.execute(*args)
-        except TrytonServerError, exception:
-            res = common.process_exception(exception, *args)
-            if not res:
-                return
+            res = RPCExecute('model', self.model, 'read', [obj_id],
+                [x[0] for x in fields], context=self.context)
+        except RPCException:
+            return
         message_str = ''
         for line in res:
             for (key, val) in fields:
                 value = str(line.get(key, False) or '/')
                 if line.get(key, False) \
                         and key in ('create_date', 'write_date'):
-                    display_format = date_format() + ' ' + HM_FORMAT
+                    display_format = date_format() + ' %H:%M:%S'
                     date = timezoned_date(line[key])
                     value = common.datetime_strftime(date, display_format)
-                message_str += val + ' ' + value +'\n'
+                message_str += val + ' ' + value + '\n'
         message_str += _('Model:') + ' ' + self.model
         message(message_str)
         return True
 
     def sig_remove(self, widget=None):
+        if not common.MODELACCESS[self.model]['delete']:
+            return
         if self.screen.current_view.view_type == 'form':
             msg = _('Are you sure to remove this record?')
         else:
@@ -321,8 +283,7 @@ class Form(SignalEvent, TabContent):
         fields = {}
         for name, field in self.screen.group.fields.iteritems():
             fields[name] = field.attrs
-        win = WinImport(self.model)
-        win.run()
+        WinImport(self.model)
 
     def sig_save_as(self, widget=None):
         while self.screen.view_to_load:
@@ -330,11 +291,11 @@ class Form(SignalEvent, TabContent):
         fields = {}
         for name, field in self.screen.group.fields.iteritems():
             fields[name] = field.attrs
-        win = WinExport(self.model, self.ids_get(),
-                context=self.context)
-        win.run()
+        WinExport(self.model, self.ids_get(), context=self.context)
 
     def sig_new(self, widget=None, autosave=True):
+        if not common.MODELACCESS[self.model]['create']:
+            return
         if autosave:
             if not self.modified_save():
                 return
@@ -343,22 +304,23 @@ class Form(SignalEvent, TabContent):
         self.activate_save()
 
     def sig_copy(self, widget=None):
+        if not common.MODELACCESS[self.model]['create']:
+            return
         if not self.modified_save():
             return
         res_ids = self.sel_ids_get()
-        ctx = self.context.copy()
-        ctx.update(rpc.CONTEXT)
-        args = ('model', self.model, 'copy', res_ids, {}, ctx)
         try:
-            new_ids = rpc.execute(*args)
-        except TrytonServerError, exception:
-            new_ids = common.process_exception(exception, *args)
-        if new_ids:
-            self.screen.load(new_ids)
-            self.message_info(_('Working now on the duplicated record(s)!'),
-                    'green')
+            new_ids = RPCExecute('model', self.model, 'copy', res_ids, {},
+                context=self.context)
+        except RPCException:
+            return
+        self.screen.load(new_ids)
+        self.message_info(_('Working now on the duplicated record(s)!'),
+            'green')
 
     def sig_save(self, widget=None):
+        if not common.MODELACCESS[self.model]['write']:
+            return
         if self.screen.save_current():
             self.message_info(_('Record saved!'), 'green')
             return True
@@ -371,36 +333,32 @@ class Form(SignalEvent, TabContent):
             return
         self.screen.display_prev()
         self.message_info('')
+        self.activate_save()
 
     def sig_next(self, widget=None):
         if not self.modified_save():
             return
         self.screen.display_next()
         self.message_info('')
+        self.activate_save()
 
     def sig_reload(self, test_modified=True):
         if not hasattr(self, 'screen'):
             return False
-        if test_modified and self.screen.modified():
-            res = sur_3b(_('This record has been modified\n' \
-                    'do you want to save it ?'))
-            if res == 'ok':
-                self.sig_save(None)
-            elif res == 'ko':
-                pass
-            else:
+        if test_modified and not self.modified_save():
                 return False
-        if self.screen.current_view.view_type == 'form':
-            self.screen.cancel_current()
-            self.screen.display()
-        else:
+        self.screen.cancel_current()
+        set_cursor = False
+        self.screen.save_tree_state(store=False)
+        if self.screen.current_view.view_type != 'form':
             obj_id = self.id_get()
             self.screen.search_filter(self.screen.screen_container.get_text())
             for record in self.screen.group:
                 if record.id == obj_id:
                     self.screen.current_record = record
-                    self.screen.display(set_cursor=True)
+                    set_cursor = True
                     break
+        self.screen.display(set_cursor=set_cursor)
         self.message_info('')
         self.activate_save()
         return True
@@ -425,6 +383,11 @@ class Form(SignalEvent, TabContent):
         if self.buttons['relate'].props.sensitive:
             self.buttons['relate'].props.active = True
 
+    def sig_search(self, widget):
+        search_container = self.screen.screen_container
+        if hasattr(search_container, 'search_entry'):
+            search_container.search_entry.grab_focus()
+
     def action_popup(self, widget):
         button, = widget.get_children()
         button.grab_focus()
@@ -460,11 +423,13 @@ class Form(SignalEvent, TabContent):
         if signal_data[0]:
             name = str(signal_data[0])
         for button_id in ('print', 'action', 'relate', 'email', 'open', 'save',
-                'switch', 'attach'):
+                'attach'):
             button = self.buttons[button_id]
             can_be_sensitive = getattr(button, '_can_be_sensitive', True)
             button.props.sensitive = (bool(signal_data[0])
                 and can_be_sensitive)
+        button_switch = self.buttons['switch']
+        button_switch.props.sensitive = self.screen.number_of_views > 1
 
         msg = name + ' / ' + str(signal_data[1])
         if signal_data[1] < signal_data[2]:
@@ -472,34 +437,36 @@ class Form(SignalEvent, TabContent):
         self.status_label.set_text(msg)
         self.message_info('')
         self.activate_save()
+        self.url_entry.set_text(self.screen.get_url())
 
     def _record_modified(self, screen, signal_data):
-        self.message_info('', color='white')
-        self.activate_save()
+        # As it is called via idle_add, the form could have been destroyed in
+        # the meantime.
+        if screen == self.screen:
+            self.activate_save()
 
     def _record_saved(self, screen, signal_data):
         self.activate_save()
+        self.update_attachment_count()
 
-    def modified_save(self, reload=True):
+    def modified_save(self):
+        self.screen.save_tree_state()
         self.screen.current_view.set_value()
         if self.screen.modified():
             value = sur_3b(
-                    _('This record has been modified\n' \
-                            'do you want to save it ?'))
+                _('This record has been modified\n'
+                    'do you want to save it ?'))
             if value == 'ok':
                 return self.sig_save(None)
-            elif value == 'ko':
-                if reload:
-                    self.sig_reload(test_modified=False)
-                return True
-            else:
-                return False
+            if value == 'ko':
+                return self.sig_reload(test_modified=False)
+            return False
         return True
 
     def sig_close(self, widget=None):
         for dialog in self.dialogs[:]:
             dialog.destroy()
-        return self.modified_save(reload=False)
+        return self.modified_save()
 
     def _action(self, action, atype):
         action = action.copy()
@@ -507,36 +474,13 @@ class Form(SignalEvent, TabContent):
             return
         record_id = self.screen.id_get()
         record_ids = self.screen.sel_ids_get()
-        if atype in ('print', 'action'):
-            email = {}
-            if 'pyson_email' in action:
-                email = self.screen.current_record.expr_eval(
-                    action['pyson_email'])
-                if not email:
-                    email = {}
-            if 'subject' not in email:
-                email['subject'] = action['name'].replace('_', '')
-            action['email'] = email
-            self.screen.display()
-        elif atype == 'relate':
-            encoder = PYSONEncoder()
-            if 'pyson_domain' in action:
-                action['pyson_domain'] = encoder.encode(
-                        self.screen.current_record.expr_eval(
-                                action['pyson_domain'], check_load=False))
-            if 'pyson_context' in action:
-                action['pyson_context'] = encoder.encode(
-                        self.screen.current_record.expr_eval(
-                            action['pyson_context'], check_load=False))
-        else:
-            raise NotImplementedError("Action type '%s' is not supported" %
-                atype)
+        action = Action.evaluate(action, atype, self.screen.current_record)
         data = {
             'model': self.screen.model_name,
             'id': record_id,
             'ids': record_ids,
         }
-        Action._exec_action(action, data, {})
+        Action._exec_action(action, data, self.context)
 
     def activate_save(self):
         self.buttons['save'].props.sensitive = self.screen.modified()
@@ -570,7 +514,7 @@ class Form(SignalEvent, TabContent):
                 tbutton.connect('toggled', self.action_popup)
                 self.tooltips.set_tip(tbutton, tooltip)
                 self.buttons[special_action] = tbutton
-                tbutton._can_be_sensitive = bool(toolbars.get(action_type))
+                tbutton._can_be_sensitive = bool(tbutton._menu.get_children())
             else:
                 tbutton = gtk.SeparatorToolItem()
             gtktoolbar.insert(tbutton, -1)
@@ -595,7 +539,18 @@ class Form(SignalEvent, TabContent):
             menuitem.connect('activate', self._popup_menu_selected, widget,
                 new_action, keyword)
             menu.add(menuitem)
-
+        if keyword == 'action':
+            menu.add(gtk.SeparatorMenuItem())
+            for plugin in plugins.MODULES:
+                for name, func in plugin.get_plugins(self.model):
+                    menuitem = gtk.MenuItem('_' + name)
+                    menuitem.set_use_underline(True)
+                    menuitem.connect('activate', lambda m, func: func({
+                                'model': self.model,
+                                'ids': self.id_get(),
+                                'id': self.id_get(),
+                                }), func)
+                    menu.add(menuitem)
         return menu
 
     def _popup_menu_selected(self, menuitem, togglebutton, action, keyword):
@@ -610,3 +565,7 @@ class Form(SignalEvent, TabContent):
 
     def _popup_menu_hide(self, menuitem, togglebutton):
         togglebutton.props.active = False
+
+    def set_cursor(self):
+        if self.screen:
+            self.screen.set_cursor(reset_view=False)
