@@ -53,7 +53,7 @@ class Screen(SignalEvent):
         if domain is None:
             domain = []
 
-        self.limit = limit or int(CONFIG['client.limit'])
+        self.limit = limit or CONFIG['client.limit']
         self.offset = 0
         super(Screen, self).__init__()
 
@@ -140,7 +140,7 @@ class Screen(SignalEvent):
                         ):
                     if name not in fields:
                         fields[name] = {
-                            'string': string,
+                            'string': string.decode('utf-8'),
                             'name': name,
                             'type': type_,
                             }
