@@ -16,6 +16,7 @@ from tryton.common import RPCExecute, RPCException, timezoned_date, \
     datetime_strftime, Tooltips
 from tryton.common.date_widget import DateEntry
 from tryton.common.placeholder_entry import PlaceholderEntry
+from tryton.common.selection import selection_shortcuts
 from tryton.translate import date_format
 
 _ = gettext.gettext
@@ -86,6 +87,7 @@ class DictSelectionEntry(DictEntry):
             lambda w: self.parent_widget._focus_out())
         widget.connect('notify::active',
             lambda w, e: self.parent_widget._focus_out())
+        selection_shortcuts(widget)
 
         # setting completion and selection
         model = gtk.ListStore(gobject.TYPE_STRING)
