@@ -307,7 +307,7 @@ class One2Many(WidgetInterface):
         if record:
             fields = self.screen.current_view.get_fields()
             if not record.validate(fields):
-                self.screen.display()
+                self.screen.display(set_cursor=True)
                 return False
             if self.screen.pre_validate and not record.pre_validate():
                 return False
@@ -328,8 +328,7 @@ class One2Many(WidgetInterface):
             if sequence:
                 self.screen.group.set_sequence(field=sequence)
 
-        if (self.screen.current_view.view_type == 'form') \
-                or self.screen.editable_get():
+        if self.screen.current_view.editable:
             self.screen.new()
             self.screen.current_view.widget.set_sensitive(True)
             update_sequence()
@@ -440,7 +439,7 @@ class One2Many(WidgetInterface):
         if field is None:
             self.screen.new_group()
             self.screen.current_record = None
-            self.screen.parent = True
+            self.screen.parent = None
             self.screen.display()
             return False
         new_group = field.get_client(record)
@@ -448,7 +447,7 @@ class One2Many(WidgetInterface):
         if id(self.screen.group) != id(new_group):
             self.screen.group = new_group
             if (self.screen.current_view.view_type == 'tree') \
-                    and self.screen.editable_get():
+                    and self.screen.current_view.editable:
                 self.screen.current_record = None
             readonly = False
             domain = []
