@@ -42,8 +42,6 @@ class AdaptModelGroup(gtk.GenericTreeModel):
     def __init__(self, group, children_field=None):
         super(AdaptModelGroup, self).__init__()
         self.group = group
-        self.last_sort = None
-        self.sort_asc = True
         self.set_property('leak_references', False)
         self.children_field = children_field
         self.__removed = None  # XXX dirty hack to allow update of has_child
@@ -335,6 +333,10 @@ class ViewList(ParserView):
     def modified(self):
         return False
 
+    @property
+    def editable(self):
+        return bool(getattr(self.widget_tree, 'editable', False))
+
     def get_fields(self):
         return [col.name for col in self.widget_tree.get_columns() if col.name]
 
@@ -449,8 +451,7 @@ class ViewList(ParserView):
         return
 
     def on_paste(self):
-        if (not hasattr(self.widget_tree, 'editable')
-                or not self.widget_tree.editable):
+        if not self.editable:
             return
 
         def unquote(value):
@@ -753,15 +754,12 @@ class ViewList(ParserView):
         elif tree_sel.get_mode() == gtk.SELECTION_MULTIPLE:
             model, paths = tree_sel.get_selected_rows()
             if model and paths:
-                iter_ = model.get_iter(paths[0])
-                record = model.get_value(iter_, 0)
+                record = model.on_get_iter(paths[0])
                 self.screen.current_record = record
             else:
                 self.screen.current_record = None
 
-        if (hasattr(self.widget_tree, 'editable')
-                and self.widget_tree.editable
-                and previous_record):
+        if self.editable and previous_record:
             def go_previous():
                 self.screen.current_record = previous_record
                 self.set_cursor()
@@ -790,8 +788,7 @@ class ViewList(ParserView):
         self.update_children()
 
     def set_value(self):
-        if hasattr(self.widget_tree, 'editable') \
-                and self.widget_tree.editable:
+        if self.editable:
             self.widget_tree.set_value()
 
     def reset(self):
@@ -814,8 +811,7 @@ class ViewList(ParserView):
             if self.store:
                 self.widget_tree.set_model(self.store)
         self.widget_tree.queue_draw()
-        if hasattr(self.widget_tree, 'editable') \
-                and self.widget_tree.editable:
+        if self.editable:
             self.set_state()
         self.update_children()
 
@@ -860,12 +856,16 @@ class ViewList(ParserView):
             path = self.store.on_get_path(self.screen.current_record)
             if self.store.get_flags() & gtk.TREE_MODEL_LIST_ONLY:
                 path = (path[0],)
-            focus_column = self.widget_tree.next_column(path)
+            focus_column = self.widget_tree.next_column(path, editable=new)
             if path[:-1]:
                 self.widget_tree.expand_to_path(path[:-1])
             self.widget_tree.scroll_to_cell(path, focus_column,
                 use_align=False)
-            self.widget_tree.set_cursor(path, focus_column, new)
+            current_path = self.widget_tree.get_cursor()[0]
+            selected_path = \
+                self.widget_tree.get_selection().get_selected_rows()[1]
+            if (current_path != path and path not in selected_path) or new:
+                self.widget_tree.set_cursor(path, focus_column, new)
 
     def sel_ids_get(self):
         def _func_sel_get(store, path, iter, ids):
@@ -899,6 +899,33 @@ class ViewList(ParserView):
                 else:
                     renderer.set_property('editable', False)
 
+    def get_selected_paths(self):
+        selection = self.widget_tree.get_selection()
+        model, rows = selection.get_selected_rows()
+        id_paths = []
+        for row in rows:
+            path = ()
+            id_path = []
+            for node in row:
+                path += (node,)
+                id_path.append(model.on_get_iter(path).id)
+            id_paths.append(id_path)
+        return id_paths
+
+    def select_nodes(self, nodes):
+        selection = self.widget_tree.get_selection()
+        if not nodes:
+            return
+        selection.unselect_all()
+        scroll = False
+        for node in nodes:
+            path = path_convert_id2pos(self.store, node)
+            if path:
+                selection.select_path(path)
+                if not scroll:
+                    self.widget_tree.scroll_to_cell(path)
+                    scroll = True
+
     def get_expanded_paths(self, starting_path=None, starting_id_path=None):
         # Use id instead of position
         # because the position may change between load
